// Simulation of discrete model
//  (multinomial model - controlled coin with memmory)
//         f( yt(t) | ut(t),yt(t-1) ),  yt,ut=1,2
// ---------------------------------------------------
[u,t,n]=file();               // find working directory
chdir(dirname(n(2)));         // set working directory
clear u t n		          // clear auxiliary variables
exec("ScIntro.sce",-1)

// PARAMETERS OF THE SIMULATION
// model parameter (conditional probabilities)
//yt(t)=1 =2  //  ut(t) yt(t-1)
// ---------------------------
th= [.2 .8    //  1    1
     .6 .4    //  1    2
     .9 .1    //  2    1
     .3 .7];  //  2    2
// E.g.  if ut(t)=2 and yt(t-1)=1 then P(yt(t)=1)=.9  etc.
// Following input
// th=[1 0;1 0; 0 1; 0 1];

nd=50;                       // number of steps
ut=(rand(1,nd,'u')>.3)+1;     // control variable P(ut=1)=.3, P(ut=2)=.7
yt(1)=1;                      // initial condition for output

// TIME LOOP OF THE SIMULATION
for t=2:nd                    
  i=2*(ut(t)-1)+yt(t-1);       // row in the model parameter
  yt(t)=(rand(1,1,'u')>th(i,1))+1;  // generation of the output
end

// RESULTS OF THE SIMULATION
subplot(211),plot(1:nd,ut,'g:.')
set(gcf(),"position",[700 100 600 500])
title('Input')
set(gca(),'data_bounds',[0 nd+1 .9 2.1])
subplot(212),plot(1:nd,yt,'b:.')
title('Output')
set(gca(),'data_bounds',[0 nd+1 .9 2.1])

