// Estimation of 2nd order regression model (uses T11)
// - least squares estimation (off-line)
// ----------------------------------------
[u,t,n]=file();               // find working directory
chdir(dirname(n(2)));         // set working directory
clear u t n		          // clear auxiliary variables
exec("ScIntro.sce",-1), mode(0)

// SIMULATION
// parameters
nd=100;                       // length of data
a=[.4 .2];                    // parameters at yt
b=[1 .2 -.5];                 // parameters at ut
k=0;                          // constant (model absolute term)
s=.1;                         // noise variance

yt(1)=1; yt(2)=3;             // initial conditions for output
ut=sin(10*%pi*(1:nd)/nd)'+.001*rand(nd,1,'n'); // input

// time loop
th=[a b k]';                  // vector of parameters
for t=3:nd
  // regression vector
  ps=[yt(t-1) yt(t-2) ut(t) ut(t-1) ut(t-2) 1]';
  // regression model
  yt(t)=th'*ps+s*rand(1,1,'norm'); 
end

// ESTIMATION
for t=3:nd
  Y(t,1)=yt(t);
  X(t,:)=[yt(t-1) yt(t-2) ut(t) ut(t-1) ut(t-2) 1];
end
th=inv(X'*X)*X'*Y;            // regression coefficients
yp=X*th;                      // prediction (for verification)
r=variance(yt-yp);           // noise variance

// Results
disp('Parameter estimates')
th,r

scf(1);                       // comparison od output and prediction
plot(1:nd,yt,1:nd,yp)
legend('optput','prediction');
title('Verification of the estimates')
