// Estimation of 2nd order regression model (uses T11)
// - Bayes on-line estimation with statistic update
// Remark: rewrite it into a single time loop
// ----------------------------------------
[u,t,n]=file();               // find working directory
chdir(dirname(n(2)));         // set working directory
clear u t n		          // clear auxiliary variables
exec("ScIntro.sce",-1)

// SIMULATION
// parameters
nd=100;                       // length of data
a=[.4 .2];                    // parameters at yt
b=[1 .2 -.5];                 // parameters at ut
k=0;                          // constant (model absolute term)
s=.1;                         // noise variance

yt(1)=1; yt(2)=3;             // initial conditions for output
ut=sin(10*%pi*(1:nd)/nd)'+.001*rand(nd,1,'n'); // input

// time loop
th=[a b k]';                  // vector of parameters
for t=3:nd
  // regression vector
  ps=[yt(t-1) yt(t-2) ut(t) ut(t-1) ut(t-2) 1]';
  // regression model
  yt(t)=th'*ps+s*rand(1,1,'norm'); 
end

// ESTIMATION
V=1e-8*eye(7,7);              // initial information matrix
for t=3:nd
  psi=[yt(t:-1:t-2); ut(t:-1:t-2); 1];       // reg. vector
  V=V+psi*psi';                              // updt of information matrix
  Vy=V(1,1); Vyp=V(2:$,1); Vp=V(2:$,2:$);    // divisioning of inf. matrix
  th(:,t)=inv(Vp+1e-8*eye(Vp))*Vyp;          // pt estimates of reg. coefficints
  r(t)=(Vy-Vyp'*inv(Vp+1e-8*eye(Vp))*Vyp)/t; // pt estimates of noise variance
end

// Results
set(scf(1),'position',[500 60 600 500])
for i=1:6
  subplot(6,1,i),plot(th(i,:))
  if i==1, title('Regression coefficients'), end
end
set(scf(2),'position',[50 60 400 200])
plot(r)
title('Noise variance')
