// Prediction with discrete model (on-line)
// Vary: - length of prediction
//       - uncertainty of the simulated model
//       - imput signal
//       - study the beginning when estimation is not finished
//         how can we secure quicker transient phase of estimation?
[u,t,n]=file();               // find working directory
chdir(dirname(n(2)));         // set working directory
clear u t n		          // clear auxiliary variables
exec("ScIntro.sce",-1),mode(0)

nd=100;                            // number of data
np=5;                              // length of prediction
th1=[0.8 0.1 0.4 0.7]';            // parameters (for y=1)
th=[th1 1-th1];                    // all parameters
u=(rand(1,nd,'u')>.3)+1;           // input
y=ones(1,nd);

// TIME LOOP
nu=1e-8*ones(4,2);
Et=zeros(4,nd-np);
for t=2:nd-np                      // time loop
  // simulation
  i=2*(u(t)-1)+y(t-1);
  y(t)=(rand(1,1,'u')>th(i,1))+1;
 
  // estimation
  i=2*(u(t)-1)+y(t-1);             // row of model matrix
  nu(i,y(t))=nu(i,y(t))+1;         // statistics update
  Eth=nu./(sum(nu,2)*ones(1,2));   // pt estimates
  Et(:,t)=Eth(:,1);

  // prediction
  yj(t)=y(t);                      // copy of past data
  for j=1:np                       // loop of prediction
    tj=t+j;
    i=2*(u(tj)-1)+yj(tj-1);        // row of model matrix
    yj(tj)=(rand(1,1,'u')>Eth(i,1))+1; // prediction
  end
  yp(tj)=yj(tj);                   // remember last prediction (on-line run)
end

// Results
disp(' Simulated parameters')
disp(th)
disp(' Estimated parameters')
disp(Eth)

subplot(121),plot(Et')
set(gcf(),'position',[500 100 1000 400])
set(gca(),"data_bounds",[0 nd+1 -.1 1.1])
subplot(122)
s=(np+3):nd;
plot(s,y(s),'.:',s,yp(s),'rx')
set(gca(),"data_bounds",[0 nd+1 .9 2.1])
legend('output','prediction');
title([string(np),'-steps ahead prediction'])

Wrong=sum(y(:)~=yp(:))
From=nd
