// Kalman as a noise filter
// - change Rw and Rv to catch properly the signal
// - Rw ... changes of the signal
// - Rv ... changes of the noise
// ----------------------------------------------------------
[u,t,n]=file();               // find working directory
chdir(dirname(n(2)));         // set working directory
clear u t n		          // clear auxiliary variables
exec SCIHOME/ScIntro.sce, mode(0)
getd()

// SIMULATION
tt=0:.1:(2*%pi);
nt=length(tt);
sd=2;                              // simulation noise
e=[sd*rand(1,nt,'n'); sd*rand(1,nt,'n')];
g=[10*cos(tt); 15*sin(tt)];        // pure signal (ellipse)
x=g+e;                             // measured noisy signal

// FILTRATION
Rz=1e6*eye(2,2);                   // state-estimate cov. matrix
Rw=.01*eye(2,2);                   // state-model cov. matrix
Rv=.1*eye(2,2);                    // output-model cov. matrix
M=[1 0                             // state-model matrices
   0 1];
A=[1 0
   0 1];
N=[0 0]';                          
F=[0 0]';
B=0;
G=0;
zt(:,1)=[0 0]';                          // initial state

for t=2:nt
  [zt(:,t),Rz,yp]=Kalman(zt(:,t-1),x(:,t),0,M,N,F,A,B,G,Rw,Rv,Rz);
end

// Results
plot(g(1,:),g(2,:),'m:')
plot(x(1,:),x(2,:),'b.')
plot(zt(1,:),zt(2,:),'r.:')
